@@ -21,10 +21,10 @@
 
 'use strict';
 const common = require('../common');
+const fixtures = require('../common/fixtures');
 const assert = require('assert');
-const path = require('path');
 const fs = require('fs');
-const filepath = path.join(common.fixturesDir, 'x.txt');
+const filepath = fixtures.path('x.txt');
 const fd = fs.openSync(filepath, 'r');
 
 const expected = Buffer.from('xyz\n');
@@ -35,8 +35,7 @@ function test(bufferAsync, bufferSync, expected) {
           0,
           expected.length,
           0,
-          common.mustCall((err, bytesRead) => {
-            assert.ifError(err);
+          common.mustSucceed((bytesRead) => {
             assert.strictEqual(bytesRead, expected.length);
             assert.deepStrictEqual(bufferAsync, expected);
           }));
@@ -53,3 +52,52 @@ test(Buffer.allocUnsafe(expected.length),
 test(new Uint8Array(expected.length),
      new Uint8Array(expected.length),
      Uint8Array.from(expected));
+
+{
+  // Reading beyond file length (3 in this case) should return no data.
+  // This is a test for a bug where reads > uint32 would return data
+  // from the current position in the file.
+  const pos = 0xffffffff + 1; // max-uint32 + 1
+  const nRead = fs.readSync(fd, Buffer.alloc(1), 0, 1, pos);
+  assert.strictEqual(nRead, 0);
+
+  fs.read(fd, Buffer.alloc(1), 0, 1, pos, common.mustSucceed((nRead) => {
+    assert.strictEqual(nRead, 0);
+  }));
+}
+
+assert.throws(() => new fs.Dir(), {
+  code: 'ERR_MISSING_ARGS',
+});
+
+assert.throws(
+  () => fs.read(fd, Buffer.alloc(1), 0, 1, 0),
+  {
+    message: 'Callback must be a function. Received undefined',
+    code: 'ERR_INVALID_CALLBACK',
+  }
+);
+
+assert.throws(
+  () => fs.read(fd, { buffer: null }, common.mustNotCall()),
+  /TypeError: Cannot read properties of null \(reading 'byteLength'\)/,
+  'throws when options.buffer is null'
+);
+
+assert.throws(
+  () => fs.readSync(fd, { buffer: null }),
+  {
+    name: 'TypeError',
+    message: 'The "buffer" argument must be an instance of Buffer, ' +
+    'TypedArray, or DataView. Received an instance of Object',
+  },
+  'throws when options.buffer is null'
+);
+
+assert.throws(
+  () => fs.read(null, Buffer.alloc(1), 0, 1, 0),
+  {
+    message: 'The "fd" argument must be of type number. Received null',
+    code: 'ERR_INVALID_ARG_TYPE',
+  }
+);
