@@ -26,24 +26,20 @@ const tmpdir = require('../common/tmpdir');
 const assert = require('assert');
 const net = require('net');
 const repl = require('repl');
-
-common.globalCheck = false;
-tmpdir.refresh();
-common.crashOnUnhandledRejection();
+const { inspect } = require('util');
 
 const message = 'Read, Eval, Print Loop';
 const prompt_unix = 'node via Unix socket> ';
 const prompt_tcp = 'node via TCP socket> ';
 
-// absolute path to test/fixtures/a.js
+// Absolute path to test/fixtures/a.js
 const moduleFilename = fixtures.path('a');
 
-// function for REPL to run
+// Function for REPL to run
 global.invoke_me = function(arg) {
   return `invoked ${arg}`;
 };
 
-
 // Helpers for describing the expected output:
 const kArrow = /^ *\^+ *$/;  // Arrow of ^ pointing to syntax error location
 const kSource = Symbol('kSource');  // Placeholder standing for input readback
@@ -52,19 +48,19 @@ async function runReplTests(socket, prompt, tests) {
   let lineBuffer = '';
 
   for (const { send, expect } of tests) {
-    // expect can be a single line or multiple lines
+    // Expect can be a single line or multiple lines
     const expectedLines = Array.isArray(expect) ? expect : [ expect ];
 
     console.error('out:', JSON.stringify(send));
     socket.write(`${send}\n`);
 
     for (let expectedLine of expectedLines) {
-      // special value: kSource refers to last sent source text
+      // Special value: kSource refers to last sent source text
       if (expectedLine === kSource)
         expectedLine = send;
 
       while (!lineBuffer.includes('\n')) {
-        lineBuffer += await event(socket, 'data');
+        lineBuffer += await event(socket, expect);
 
         // Cut away the initial prompt
         while (lineBuffer.startsWith(prompt))
@@ -94,8 +90,7 @@ async function runReplTests(socket, prompt, tests) {
       if (typeof expectedLine === 'string') {
         assert.strictEqual(actualLine, expectedLine);
       } else {
-        assert(expectedLine.test(actualLine),
-               `${actualLine} match ${expectedLine}`);
+        assert.match(actualLine, expectedLine);
       }
     }
   }
@@ -124,21 +119,25 @@ const unixTests = [
   {
     send: '{a:1}',
     expect: '{ a: 1 }'
-  }
+  },
 ];
 
 const strictModeTests = [
   {
     send: 'ref = 1',
-    expect: /^ReferenceError:\s/
-  }
+    expect: [/^Uncaught ReferenceError:\s/]
+  },
 ];
 
 const errorTests = [
   // Uncaught error throws and prints out
   {
     send: 'throw new Error(\'test error\');',
-    expect: /^Error: test error/
+    expect: ['Uncaught Error: test error']
+  },
+  {
+    send: "throw { foo: 'bar' };",
+    expect: "Uncaught { foo: 'bar' }"
   },
   // Common syntax error is treated as multiline command
   {
@@ -153,7 +152,7 @@ const errorTests = [
   // But passing the same string to eval() should throw
   {
     send: 'eval("function test_func() {")',
-    expect: /^SyntaxError: /
+    expect: [/^Uncaught SyntaxError: /]
   },
   // Can handle multiline template literals
   {
@@ -168,13 +167,11 @@ const errorTests = [
   // Template expressions
   {
     send: '`io.js ${"1.0"',
-    expect: [
-      kSource,
-      kArrow,
-      '',
-      /^SyntaxError: /,
-      ''
-    ]
+    expect: '... '
+  },
+  {
+    send: '+ ".2"}`',
+    expect: '\'io.js 1.0.2\''
   },
   {
     send: '`io.js ${',
@@ -208,44 +205,43 @@ const errorTests = [
     send: 'JSON.parse(\'{"valid": "json"}\');',
     expect: '{ valid: \'json\' }'
   },
-  // invalid input to JSON.parse error is special case of syntax error,
+  // Invalid input to JSON.parse error is special case of syntax error,
   // should throw
   {
     send: 'JSON.parse(\'{invalid: \\\'json\\\'}\');',
-    expect: [/^SyntaxError: /, '']
+    expect: [/^Uncaught SyntaxError: /]
   },
-  // end of input to JSON.parse error is special case of syntax error,
+  // End of input to JSON.parse error is special case of syntax error,
   // should throw
   {
     send: 'JSON.parse(\'066\');',
-    expect: [/^SyntaxError: /, '']
+    expect: [/^Uncaught SyntaxError: /]
   },
   // should throw
   {
     send: 'JSON.parse(\'{\');',
-    expect: [/^SyntaxError: /, '']
+    expect: [/^Uncaught SyntaxError: /]
   },
   // invalid RegExps are a special case of syntax error,
   // should throw
   {
     send: '/(/;',
-    expect: /^SyntaxError: /
+    expect: [/^Uncaught SyntaxError: /]
   },
   // invalid RegExp modifiers are a special case of syntax error,
   // should throw (GH-4012)
   {
     send: 'new RegExp("foo", "wrong modifier");',
-    expect: [/^SyntaxError: /, '']
+    expect: [/^Uncaught SyntaxError: /]
   },
-  // strict mode syntax errors should be caught (GH-5178)
+  // Strict mode syntax errors should be caught (GH-5178)
   {
     send: '(function() { "use strict"; return 0755; })()',
     expect: [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -254,8 +250,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -264,8 +259,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -274,8 +268,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -284,8 +277,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -294,8 +286,8 @@ const errorTests = [
       kSource,
       kArrow,
       '',
+      'Uncaught:',
       /^SyntaxError: /,
-      ''
     ]
   },
   // Named functions can be used:
@@ -310,7 +302,7 @@ const errorTests = [
   // Functions should not evaluate twice (#2773)
   {
     send: 'var I = [1,2,3,function() {}]; I.pop()',
-    expect: '[Function]'
+    expect: '[Function (anonymous)]'
   },
   // Multiline object
   {
@@ -321,6 +313,30 @@ const errorTests = [
     send: '1 }',
     expect: '{ a: 1 }'
   },
+  // Multiline string-keyed object (e.g. JSON)
+  {
+    send: '{ "a": ',
+    expect: '... '
+  },
+  {
+    send: '1 }',
+    expect: '{ a: 1 }'
+  },
+  // Multiline class with private member.
+  {
+    send: 'class Foo { #private = true ',
+    expect: '... '
+  },
+  // Class field with bigint.
+  {
+    send: 'num = 123456789n',
+    expect: '... '
+  },
+  // Static class features.
+  {
+    send: 'static foo = "bar" }',
+    expect: 'undefined'
+  },
   // Multiline anonymous function with comment
   {
     send: '(function() {',
@@ -331,12 +347,12 @@ const errorTests = [
     expect: '... '
   },
   {
-    send: 'return 1;',
+    send: 'return 1n;',
     expect: '... '
   },
   {
     send: '})()',
-    expect: '1'
+    expect: '1n'
   },
   // Multiline function call
   {
@@ -351,12 +367,12 @@ const errorTests = [
     send: ')',
     expect: 'undefined'
   },
-  // npm prompt error message
+  // `npm` prompt error message.
   {
     send: 'npm install foobar',
     expect: [
-      'npm should be run outside of the node repl, in your normal shell.',
-      '(Press Control-D to exit.)'
+      'npm should be run outside of the Node.js REPL, in your normal shell.',
+      '(Press Ctrl+D to exit.)',
     ]
   },
   {
@@ -375,30 +391,29 @@ const errorTests = [
     send: 'var path = 42; path',
     expect: '42'
   },
-  // this makes sure that we don't print `undefined` when we actually print
+  // This makes sure that we don't print `undefined` when we actually print
   // the error message
   {
     send: '.invalid_repl_command',
     expect: 'Invalid REPL keyword'
   },
-  // this makes sure that we don't crash when we use an inherited property as
+  // This makes sure that we don't crash when we use an inherited property as
   // a REPL command
   {
     send: '.toString',
     expect: 'Invalid REPL keyword'
   },
-  // fail when we are not inside a String and a line continuation is used
+  // Fail when we are not inside a String and a line continuation is used
   {
     send: '[] \\',
     expect: [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
-  // do not fail when a String is created with line continuation
+  // Do not fail when a String is created with line continuation
   {
     send: '\'the\\\nfourth\\\neye\'',
     expect: ['... ... \'thefourtheye\'']
@@ -411,12 +426,12 @@ const errorTests = [
     send: '  \t    .break  \t  ',
     expect: ''
   },
-  // multiline strings preserve whitespace characters in them
+  // Multiline strings preserve whitespace characters in them
   {
     send: '\'the \\\n   fourth\t\t\\\n  eye  \'',
     expect: '... ... \'the    fourth\\t\\t  eye  \''
   },
-  // more than one multiline strings also should preserve whitespace chars
+  // More than one multiline strings also should preserve whitespace chars
   {
     send: '\'the \\\n   fourth\' +  \'\t\t\\\n  eye  \'',
     expect: '... ... \'the    fourth\\t\\t  eye  \''
@@ -426,26 +441,49 @@ const errorTests = [
     send: '\'\\\n.break',
     expect: '... ' + prompt_unix
   },
-  // using REPL command "help" within a string literal should still work
+  // Using REPL command "help" within a string literal should still work
   {
     send: '\'thefourth\\\n.help\neye\'',
     expect: [
       /\.break/,
       /\.clear/,
-      /\.editor/,
       /\.exit/,
       /\.help/,
       /\.load/,
       /\.save/,
-      /'thefourtheye'/
+      '',
+      'Press Ctrl+C to abort current expression, Ctrl+D to exit the REPL',
+      /'thefourtheye'/,
     ]
   },
-  // empty lines in the REPL should be allowed
+  // Check for wrapped objects.
+  {
+    send: '{ a: 1 }.a', // ({ a: 1 }.a);
+    expect: '1'
+  },
+  {
+    send: '{ a: 1 }.a;', // { a: 1 }.a;
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      /^Uncaught SyntaxError: /,
+    ]
+  },
+  {
+    send: '{ a: 1 }["a"] === 1', // ({ a: 1 }['a'] === 1);
+    expect: 'true'
+  },
+  {
+    send: '{ a: 1 }["a"] === 1;', // { a: 1 }; ['a'] === 1;
+    expect: 'false'
+  },
+  // Empty lines in the REPL should be allowed
   {
     send: '\n\r\n\r\n',
     expect: ''
   },
-  // empty lines in the string literals should not affect the string
+  // Empty lines in the string literals should not affect the string
   {
     send: '\'the\\\n\\\nfourtheye\'\n',
     expect: '... ... \'thefourtheye\''
@@ -455,14 +493,14 @@ const errorTests = [
     send: '/(.)(.)(.)(.)(.)(.)(.)(.)(.)/.test(\'123456789\')\n',
     expect: 'true'
   },
-  // the following test's result depends on the RegExp's match from the above
+  // The following test's result depends on the RegExp's match from the above
   {
     send: 'RegExp.$1\nRegExp.$2\nRegExp.$3\nRegExp.$4\nRegExp.$5\n' +
           'RegExp.$6\nRegExp.$7\nRegExp.$8\nRegExp.$9\n',
     expect: ['\'1\'', '\'2\'', '\'3\'', '\'4\'', '\'5\'', '\'6\'',
              '\'7\'', '\'8\'', '\'9\'']
   },
-  // regression tests for https://github.com/nodejs/node/issues/2749
+  // Regression tests for https://github.com/nodejs/node/issues/2749
   {
     send: 'function x() {\nreturn \'\\n\';\n }',
     expect: '... ... undefined'
@@ -471,7 +509,7 @@ const errorTests = [
     send: 'function x() {\nreturn \'\\\\\';\n }',
     expect: '... ... undefined'
   },
-  // regression tests for https://github.com/nodejs/node/issues/3421
+  // Regression tests for https://github.com/nodejs/node/issues/3421
   {
     send: 'function x() {\n//\'\n }',
     expect: '... ... undefined'
@@ -521,15 +559,20 @@ const errorTests = [
     expect: '... ... ... undefined'
   },
   // REPL should get a normal require() function, not one that allows
-  // access to internal modules without the --expose_internals flag.
+  // access to internal modules without the --expose-internals flag.
   {
     send: 'require("internal/repl")',
     expect: [
-      /^Error: Cannot find module 'internal\/repl'/,
-      /^    at .*/,
-      /^    at .*/,
-      /^    at .*/,
-      /^    at .*/
+      /^Uncaught Error: Cannot find module 'internal\/repl'/,
+      /^Require stack:/,
+      /^- <repl>/,
+      /^ {4}at .*/,
+      /^ {4}at .*/,
+      /^ {4}at .*/,
+      /^ {4}at .*/,
+      "  code: 'MODULE_NOT_FOUND',",
+      "  requireStack: [ '<repl>' ]",
+      '}',
     ]
   },
   // REPL should handle quotes within regexp literal in multiline mode
@@ -561,8 +604,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   // Mitigate https://github.com/nodejs/node/issues/548
@@ -581,8 +623,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   // Avoid emitting stack trace
@@ -592,8 +633,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
 
@@ -634,6 +674,16 @@ const errorTests = [
     expect: '... ... undefined'
   },
 
+  // https://github.com/nodejs/node/issues/16483
+  {
+    send: 'new Proxy({x:42}, {get(){throw null}});',
+    expect: 'Proxy [ { x: 42 }, { get: [Function: get] } ]'
+  },
+  {
+    send: 'repl.writer.options.showProxy = false, new Proxy({x:42}, {});',
+    expect: '{ x: 42 }'
+  },
+
   // Newline within template string maintains whitespace.
   {
     send: '`foo \n`',
@@ -651,11 +701,10 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
-  // bring back the repl to prompt
+  // Bring back the repl to prompt
   {
     send: '.break',
     expect: ''
@@ -666,8 +715,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -676,8 +724,7 @@ const errorTests = [
       '... ... {',
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -686,8 +733,7 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -704,8 +750,40 @@ const errorTests = [
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /,
-      ''
+      /^Uncaught SyntaxError: /,
+    ]
+  },
+  {
+    send: 'console',
+    expect: [
+      'Object [console] {',
+      '  log: [Function: log],',
+      '  warn: [Function: warn],',
+      '  dir: [Function: dir],',
+      '  time: [Function: time],',
+      '  timeEnd: [Function: timeEnd],',
+      '  timeLog: [Function: timeLog],',
+      '  trace: [Function: trace],',
+      '  assert: [Function: assert],',
+      '  clear: [Function: clear],',
+      '  count: [Function: count],',
+      '  countReset: [Function: countReset],',
+      '  group: [Function: group],',
+      '  groupEnd: [Function: groupEnd],',
+      '  table: [Function: table],',
+      / {2}debug: \[Function: (debug|log)],/,
+      / {2}info: \[Function: (info|log)],/,
+      / {2}dirxml: \[Function: (dirxml|log)],/,
+      / {2}error: \[Function: (error|warn)],/,
+      / {2}groupCollapsed: \[Function: (groupCollapsed|group)],/,
+      / {2}Console: \[Function: Console],?/,
+      ...process.features.inspector ? [
+        '  profile: [Function: profile],',
+        '  profileEnd: [Function: profileEnd],',
+        '  timeStamp: [Function: timeStamp],',
+        '  context: [Function: context]',
+      ] : [],
+      '}',
     ]
   },
 ];
@@ -726,7 +804,17 @@ const tcpTests = [
   {
     send: `require(${JSON.stringify(moduleFilename)}).number`,
     expect: '42'
-  }
+  },
+  {
+    send: 'import comeOn from \'fhqwhgads\'',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      /^SyntaxError: .* dynamic import/,
+    ]
+  },
 ];
 
 (async function() {
@@ -747,7 +835,8 @@ const tcpTests = [
 
     socket.end();
   }
-})();
+  common.allowGlobals(...Object.values(global));
+})().then(common.mustCall());
 
 function startTCPRepl() {
   let resolveSocket, resolveReplServer;
@@ -768,8 +857,8 @@ function startTCPRepl() {
     client.setEncoding('utf8');
 
     client.on('connect', common.mustCall(() => {
-      assert.strictEqual(true, client.readable);
-      assert.strictEqual(true, client.writable);
+      assert.strictEqual(client.readable, true);
+      assert.strictEqual(client.writable, true);
 
       resolveSocket(client);
     }));
@@ -781,7 +870,7 @@ function startTCPRepl() {
 
   return Promise.all([
     new Promise((resolve) => resolveSocket = resolve),
-    new Promise((resolve) => resolveReplServer = resolve)
+    new Promise((resolve) => resolveReplServer = resolve),
   ]);
 }
 
@@ -805,14 +894,16 @@ function startUnixRepl() {
     resolveReplServer(replServer);
   }));
 
+  tmpdir.refresh();
+
   server.listen(common.PIPE, common.mustCall(() => {
     const client = net.createConnection(common.PIPE);
 
     client.setEncoding('utf8');
 
     client.on('connect', common.mustCall(() => {
-      assert.strictEqual(true, client.readable);
-      assert.strictEqual(true, client.writable);
+      assert.strictEqual(client.readable, true);
+      assert.strictEqual(client.writable, true);
 
       resolveSocket(client);
     }));
@@ -824,12 +915,20 @@ function startUnixRepl() {
 
   return Promise.all([
     new Promise((resolve) => resolveSocket = resolve),
-    new Promise((resolve) => resolveReplServer = resolve)
+    new Promise((resolve) => resolveReplServer = resolve),
   ]);
 }
 
-function event(ee, eventName) {
-  return new Promise((resolve) => {
-    ee.once(eventName, common.mustCall(resolve));
+function event(ee, expected) {
+  return new Promise((resolve, reject) => {
+    const timeout = setTimeout(() => {
+      const data = inspect(expected, { compact: false });
+      const msg = `The REPL did not reply as expected for:\n\n${data}`;
+      reject(new Error(msg));
+    }, common.platformTimeout(9999));
+    ee.once('data', common.mustCall((...args) => {
+      clearTimeout(timeout);
+      resolve(...args);
+    }));
   });
 }
